import * as React from "react";
import CalendarDayView from "./CalendarDayView";
import CalendarYearView from "./CalendarYearView";
import CalendarMonthView from "./CalendarMonthView";
import CalendarSearsonView from "./CalendarSearsonView";
import { Animate } from "../animate/index";
import * as Immutable from "immutable";
import { SvgIcon } from "../my-icon/index";
import { getLastYear } from './objectFn';
var calendarType;
(function (calendarType) {
    calendarType[calendarType["year"] = 1] = "year";
    calendarType[calendarType["searson"] = 2] = "searson";
    calendarType[calendarType["month"] = 3] = "month";
    calendarType[calendarType["day"] = 4] = "day";
})(calendarType || (calendarType = {}));
export default class CalendarView extends React.PureComponent {
    constructor() {
        super(...arguments);
        this.changeSelTimeItme = (viewIndex) => {
            const { changeBasicState, rotate, fixProps: { style } } = this.props;
            changeBasicState("selTimeArr", function (states) {
                const { year, month, day, searson } = states.showTimeArr.get(viewIndex).toJS();
                let selTimeArr = states.selTimeArr;
                if (!selTimeArr.size) {
                    selTimeArr = Immutable.fromJS([{ year: "" }]);
                }
                switch (rotate) {
                    case calendarType.day:
                        selTimeArr = selTimeArr.update(viewIndex, map => {
                            return map.withMutations(node => node
                                .set("year", year)
                                .set("month", month)
                                .set("day", day)
                                .set("searson", Math.ceil(month / 3)));
                        });
                        break;
                    case calendarType.searson: {
                        selTimeArr = selTimeArr.update(viewIndex, map => {
                            return map.withMutations(node => node
                                .set("year", year)
                                .set("month", searson * 3 - 2)
                                .set("searson", searson));
                        });
                        break;
                    }
                    case calendarType.year: {
                        selTimeArr = selTimeArr.update(viewIndex, map => {
                            return map.withMutations(node => node.set("year", year));
                        });
                        break;
                    }
                    case calendarType.month: {
                        selTimeArr = selTimeArr.update(viewIndex, map => {
                            return map.withMutations(node => node
                                .set("year", year)
                                .set("month", month)
                                .set("searson", Math.ceil(month / 3)));
                        });
                        break;
                    }
                    default:
                        selTimeArr = selTimeArr;
                        break;
                }
                if (style === 2 && !states.selTimeArr.getIn([0, "year"])) {
                    selTimeArr = selTimeArr.set(1 - viewIndex, selTimeArr.get(viewIndex));
                }
                return selTimeArr;
            });
        };
        this.clickSelHandle = (e) => {
            const dataset = e.currentTarget.dataset;
            const type = dataset.sign;
            const num = ~~dataset.num;
            const { viewIndex, changeBasicState, fixProps: { style } } = this.props;
            const flag = { [type]: num };
            const _showTimeobj = Object.assign({}, this.props.showTimeObj.toJS(), flag);
            const startGtEnd = changeBasicState("selTimeArr", (state) => state.selTimeArr, {
                viewIndex,
                showTimeObj: _showTimeobj
            });
            if (startGtEnd) {
                return;
            }
            changeBasicState("showTimeArr", function (state) {
                let showTimeArr = state.showTimeArr.setIn([viewIndex, type], num);
                if (style === 2 && !state.selTimeArr.getIn([0, "year"])) {
                    showTimeArr = showTimeArr.set(1 - viewIndex, showTimeArr.get(0));
                }
                return showTimeArr;
            });
            const { rotate, showViewArr } = this.props;
            this.changeSelTimeItme(viewIndex);
            const panelRotate = showViewArr.findIndex(val => val === "fadeIn");
            if (rotate !== panelRotate) {
                changeBasicState("showViewArr", function () {
                    let animationArr = new Array(5).fill("fadeOut");
                    animationArr[rotate] = "fadeIn";
                    return animationArr;
                });
            }
        };
        this.changeView = (e) => {
            const type = e.currentTarget.dataset.sign;
            const { rotate } = this.props;
            this.props.changeBasicState("showViewArr", function (states) {
                const preIndex = states.showViewArr.findIndex(val => val === "fadeIn");
                let showViewArr = [...states.showViewArr];
                showViewArr[preIndex] = "fadeOut";
                if (preIndex === rotate) {
                    showViewArr[calendarType[type]] = "fadeIn";
                }
                else {
                    showViewArr[rotate] = "fadeIn";
                }
                return showViewArr;
            });
        };
        this.changeRotate = (e) => {
            const dom = e.currentTarget;
            if (dom.classList.contains("active")) {
                return;
            }
            const id = (+e.currentTarget.dataset.id);
            this.props.changeBasicState("rotate", function () {
                return id;
            });
            if (this.props.selTimeObj.get("year")) {
                this.props.changeBasicState("showTimeArr", function (state) {
                    return state.selTimeArr;
                });
            }
            this.props.changeBasicState("pannelLastYear", function (state) {
                return state.showTimeArr.map(val => {
                    return getLastYear(val.get("year"));
                });
            });
            if (id === 4) {
                this.props.changeBasicState("selTimeArr", function (state) {
                    return state.selTimeArr.map(val => {
                        return val.set("day", 1);
                    });
                });
            }
        };
        this.controlBtnHandle = (e) => {
            const dataset = e.currentTarget.dataset;
            const type = dataset.sign;
            const curViewIndex = +dataset.curviewindex;
            curViewIndex === calendarType.day
                ? this.updatePanelDays(type)
                : this.updatePanelYears(type);
        };
        this.changeTime = (e) => {
            const name = e.currentTarget.name;
            const viewIndex = +e.currentTarget.dataset.viewindex;
            const { changeBasicState } = this.props;
            let value = ~~e.currentTarget.value;
            value = name === "hour" ? (value > 23 ? 23 : value) : (value > 59 ? 59 : value);
            const flag = { [name]: value };
            const _showTimeobj = Object.assign({}, this.props.selTimeObj.toJS(), flag);
            console.log(flag, _showTimeobj);
            const startGtEnd = changeBasicState("selTimeArr", (state) => state.selTimeArr, {
                viewIndex,
                showTimeObj: _showTimeobj
            });
            if (startGtEnd) {
                return;
            }
            changeBasicState("selTimeArr", function (states) {
                let selTimeArr = states.selTimeArr;
                if (!selTimeArr.size) {
                    selTimeArr = Immutable.fromJS([{ year: "" }]);
                }
                selTimeArr = selTimeArr.update(viewIndex, map => {
                    return map.withMutations(node => {
                        return node.set(name, value);
                    });
                });
                console.log(selTimeArr.toJS(), "更新");
                return selTimeArr;
            });
        };
    }
    updatePanelYears(movePre) {
        const { viewIndex } = this.props;
        this.props.changeBasicState("pannelLastYear", function (state) {
            let lastYear = state.pannelLastYear.get(viewIndex);
            lastYear = movePre === "back" ? lastYear - 10 : lastYear + 1;
            let index = lastYear % 10;
            index = index === 0 ? 10 : index;
            lastYear = lastYear - index + 10;
            return state.pannelLastYear.set(viewIndex, lastYear);
        });
    }
    updatePanelDays(movePre) {
        const { changeBasicState, viewIndex, showTimeObj } = this.props;
        const year = showTimeObj.get("year"), month = showTimeObj.get("month");
        let updataMon = 1, updataYear = 1;
        switch (movePre) {
            case "back":
                updataMon = month - 1 === 0 ? 12 : month - 1;
                updataYear = month - 1 === 0 ? year - 1 : year;
                break;
            case "next":
                updataMon = month + 1 === 13 ? 1 : month + 1;
                updataYear = month + 1 === 13 ? year + 1 : year;
                break;
        }
        changeBasicState("showTimeArr", (states) => {
            return states.showTimeArr.updateIn([viewIndex], (map) => {
                return map.withMutations((map) => map
                    .set("year", updataYear)
                    .set("month", updataMon)
                    .set("searson", Math.ceil(month / 3)));
            });
        });
    }
    getRotateCombo(arr) {
        const { rotate } = this.props;
        return arr.map((val, index) => {
            const active = index === rotate - 1;
            const id = index + 1;
            return (React.createElement("li", { className: (active && "active") || "", "data-id": id, onClick: this.changeRotate, key: index }, val));
        });
    }
    getSelTime() {
        const { showTimeObj } = this.props;
        const { rotate } = this.props;
        if (rotate === calendarType.day) {
            const year = showTimeObj.get("year");
            const month = `${showTimeObj.get("month")}`.padStart(2, "0");
            return (React.createElement("div", null,
                React.createElement("i", { className: "fa fa-calendar" }),
                "\u00A0",
                React.createElement("span", { "data-sign": "year", onClick: this.changeView },
                    year,
                    "/"),
                React.createElement("span", { "data-sign": "month", onClick: this.changeView }, month)));
        }
        else if (rotate !== calendarType.year) {
            return (React.createElement("div", null,
                React.createElement("i", { className: "fa fa-calendar" }),
                "\u00A0",
                React.createElement("span", { "data-sign": "year", onClick: this.changeView },
                    showTimeObj.get("year"),
                    "\u5E74")));
        }
        else {
            return undefined;
        }
    }
    render() {
        const { curTime, selTimeObj, viewIndex, fixProps: { noChangeRotate, time }, rotate, showViewArr, showTimeObj, lastYear } = this.props;
        const curViewInde = showViewArr.findIndex(val => val === "fadeIn");
        const showMoveBtn = curViewInde === calendarType.day ||
            curViewInde === calendarType.year;
        let duration = new Array(5).fill(0);
        duration[curViewInde] = 400;
        const arr = ["年", "季", "月", "日"];
        const secondViewName = viewIndex === 1 ? "calendar-view2" : "";
        return (React.createElement("div", { className: `${secondViewName} g-calendar-view` },
            React.createElement("div", { className: "m-viewOpt" },
                React.createElement("div", { style: { position: "relative", }, className: noChangeRotate ? "" : "can-hover" },
                    React.createElement("span", { className: "curViewType " }, arr[curViewInde - 1]),
                    React.createElement("ul", { className: "m-rotate-drop" }, this.getRotateCombo(arr))),
                React.createElement("div", { className: "m-timeSel" }, this.getSelTime()),
                React.createElement("div", { className: "m-moveBtns " + (showMoveBtn ? "" : "hideMoveBtn") },
                    React.createElement("button", { className: "s-btn", onClick: this.controlBtnHandle, "data-sign": "back", "data-curviewindex": curViewInde },
                        React.createElement(SvgIcon, { className: "arrow-pre" }),
                        React.createElement("i", { className: "fa fa-backward" })),
                    React.createElement("button", { className: "s-btn", onClick: this.controlBtnHandle, "data-sign": "next", "data-curviewindex": curViewInde },
                        React.createElement(SvgIcon, { className: "arrow-next" })))),
            React.createElement("div", { className: "m-calendar-view" },
                React.createElement(Animate, { animation: showViewArr[calendarType.day], duration: duration[calendarType.day] },
                    React.createElement(CalendarDayView, { curTime: curTime, selTimeObj: selTimeObj, showTimeObj: showTimeObj, clickSelHandle: this.clickSelHandle, time: time, changeTime: this.changeTime, viewIndex: viewIndex })),
                React.createElement(Animate, { animation: showViewArr[calendarType.year], duration: duration[calendarType.year] },
                    React.createElement(CalendarYearView, { curTime: curTime, selTimeObj: selTimeObj, showTimeObj: showTimeObj, clickSelHandle: this.clickSelHandle, rotate: rotate, lastYear: lastYear })),
                React.createElement(Animate, { animation: showViewArr[calendarType.month], duration: duration[calendarType.month] },
                    React.createElement(CalendarMonthView, { curTime: curTime, selTimeObj: selTimeObj, showTimeObj: showTimeObj, clickSelHandle: this.clickSelHandle, rotate: rotate })),
                React.createElement(Animate, { animation: showViewArr[calendarType.searson], duration: duration[calendarType.searson] },
                    React.createElement(CalendarSearsonView, { curTime: curTime, selTimeObj: selTimeObj, showTimeObj: showTimeObj, clickSelHandle: this.clickSelHandle })))));
    }
}
