import * as React from "react";
import CrashPage from './CrashPage';
import notice from '../toast/index';
export default class ErrorBoundary extends React.Component {
    constructor(props) {
        super(props);
        this.state = {
            hasError: false,
            preInit: this.props.init
        };
        this.getHasError = () => {
            return this.state.hasError;
        };
        this.reload = () => {
            this.setState(() => {
                return {
                    hasError: false
                };
            }, () => {
                notice.clear();
            });
        };
        const { bindGetHasError } = props;
        if (bindGetHasError) {
            bindGetHasError(this.getHasError);
        }
    }
    static getDerivedStateFromProps(nextProps, preState) {
        const newInit = nextProps.init;
        if (newInit !== preState.preInit) {
            return {
                hasError: newInit.init,
                preInit: nextProps.init
            };
        }
        else {
            return null;
        }
    }
    static getDerivedStateFromError(_error) {
        return { hasError: true };
    }
    componentDidCatch(_error, _info) {
        notice.add("数据操作出错，点击按钮重新加载页面！", "warn", true);
    }
    render() {
        const { type, msg } = this.props;
        if (this.state.hasError) {
            return React.createElement(CrashPage, { reloadFn: this.reload, type: type, msg: msg });
        }
        else {
            return this.props.children;
        }
    }
}
