import * as React from "react";
import { Animate } from "../animate/index";
import { SvgIcon } from "../my-icon/index";
import { Button } from "../button/index";
class Search extends React.PureComponent {
    constructor() {
        super(...arguments);
        this.state = {
            searching: false,
            keyword: this.props.defaultVal || "",
            preField: this.props.field
        };
        this.toggleSearch = () => {
            const { keyword } = this.state;
            if (!keyword) {
                return;
            }
            const { field, searchHandle } = this.props;
            this.setState({
                searching: true,
            });
            searchHandle(keyword, field);
        };
        this.closeSearch = () => {
            this.setState({
                searching: false,
                keyword: "",
            });
            const { closeHandle, field } = this.props;
            if (closeHandle) {
                closeHandle(field);
            }
        };
        this.keyPress = (e) => {
            if (e.key === "Enter") {
                this.toggleSearch();
            }
        };
        this.changeFn = (e) => {
            const dom = e.currentTarget;
            const value = dom.value.trim();
            this.setState({
                keyword: value,
            });
        };
    }
    static getDerivedStateFromProps(nextProps, preState) {
        if (nextProps.field !== preState.preField) {
            const { closeHandle, field } = nextProps;
            if (closeHandle) {
                closeHandle(field);
            }
            return {
                searching: false,
                keyword: "",
                preField: nextProps.field
            };
        }
        else {
            return null;
        }
    }
    render() {
        const { searching, keyword } = this.state;
        const { tip, width, children } = this.props;
        const styleObj = width ? { width } : undefined;
        const jBtn = children ? (React.createElement(Button, { handle: this.toggleSearch }, children)) : undefined;
        const searchIcon = children ? undefined : (React.createElement("span", { className: "j-search-icon", onClick: this.toggleSearch },
            React.createElement(SvgIcon, { className: "search" })));
        const right = children ? 0.5 : 1.6;
        return (React.createElement("div", { className: "g-search" },
            React.createElement("div", { className: "m-search", style: styleObj },
                React.createElement("input", { type: "text", className: "s-inp", value: keyword, placeholder: tip, onChange: this.changeFn, onKeyDown: this.keyPress }),
                React.createElement(Animate, { animation: searching ? "fadeIn" : "fadeOut", spanWrapEle: true },
                    React.createElement("span", { className: "m-search-close", style: { right: `${right}em`, }, onClick: this.closeSearch },
                        React.createElement(SvgIcon, { className: "close" }))),
                searchIcon),
            jBtn));
    }
}
Search.defaultProps = {
    tip: "查询结果...",
};
export default Search;
