import * as React from "react";
import { fnUtil } from "../util/jsUtil";
import { tween } from "../animate/tween";
class ScrollBox extends React.PureComponent {
    constructor(props) {
        super(props);
        this.moveBar = React.createRef();
        this.state = {
            showBar: !!this.props.keepBarShow,
            moveBarH: 0,
        };
        this.scrollBoxH = 0;
        this.scrollMainH = 0;
        this.timer = 0;
        this.upTimer = 0;
        this.scrollToTop = (top) => {
            const { moveBarH } = this.state;
            const scrollMain = this.moveBar.current.parentElement
                .previousElementSibling;
            let h = top;
            const maxH = this.scrollMainH - this.scrollBoxH;
            if (maxH <= 0) {
                return;
            }
            h = h < 0 ? 0 : h > maxH ? maxH : h;
            const factor = (this.scrollBoxH - moveBarH) / maxH;
            scrollMain.style.top = `${-h}px`;
            this.moveBar.current.style.top = `${h * factor}px`;
        };
        this.barMove = (e) => {
            const { moveBarH } = this.state;
            const dom = this.moveBar.current;
            const scrollMain = dom.parentElement
                .previousElementSibling;
            const startH = +dom.dataset.top;
            const moveDistance = e.clientY - +dom.dataset.y;
            const maxH = this.scrollBoxH - moveBarH;
            let h = startH + moveDistance;
            h = h < 0 ? 0 : h > maxH ? maxH : h;
            const factor = (this.scrollMainH - this.scrollBoxH) / maxH;
            dom.style.top = `${h}px`;
            scrollMain.style.top = `${-h * factor}px`;
        };
        this.barMoveEven = fnUtil.throttle(this.barMove, 100);
        this.upDateInit = (defaultH) => {
            const barInner = this.moveBar.current;
            const scrollMain = barInner.parentElement
                .previousElementSibling;
            const curScrollMainH = scrollMain.clientHeight;
            if (!defaultH && curScrollMainH === this.scrollMainH) {
                return;
            }
            this.scrollBoxH = defaultH
                ? defaultH
                : barInner.parentElement.parentElement.clientHeight;
            this.scrollMainH = curScrollMainH;
            if (curScrollMainH < this.scrollBoxH) {
                scrollMain.style.top = "0";
                this.setState({
                    showBar: false,
                });
                return;
            }
            let moveBarH = Math.ceil((this.scrollBoxH / curScrollMainH) * this.scrollBoxH);
            moveBarH = Math.max(16, moveBarH);
            const top = scrollMain.style.top || "0";
            let curScrollMainTop = -parseInt(top, 0);
            if (curScrollMainTop + this.scrollBoxH > curScrollMainH) {
                curScrollMainTop = curScrollMainH - this.scrollBoxH;
                scrollMain.style.top = `${-curScrollMainTop}px`;
            }
            const maxH = this.scrollMainH - this.scrollBoxH;
            const factor = (this.scrollBoxH - moveBarH) / maxH;
            barInner.style.top = `${factor * curScrollMainTop}px`;
            this.animateScroll(this.state.moveBarH, moveBarH, () => {
                this.setState({
                    moveBarH,
                    showBar: true,
                });
            });
        };
        this.barClick = (e) => {
            e.currentTarget.dataset.y = `${e.clientY}`;
            const top = parseInt(e.currentTarget.style.top || "0", 0);
            e.currentTarget.dataset.top = `${top}`;
            e.currentTarget.parentElement.parentElement.addEventListener("mousemove", this.barMoveEven);
        };
        this.cancelBarMove = () => {
            this.moveBar.current.parentElement.parentElement.removeEventListener("mousemove", this.barMoveEven);
        };
        this.mouseOver = (e) => {
            const dom = e.currentTarget;
            if (!dom.firstElementChild) {
                return;
            }
            if (this.scrollMainH > this.scrollBoxH) {
                this.setState({
                    showBar: true,
                });
            }
        };
        this.mouseLeave = () => {
            this.cancelBarMove();
            if (this.props.bindIntiScroll) {
                return;
            }
            this.setState({
                showBar: false,
            });
        };
        this.mainMousewheel = (e) => {
            this.cancelBarMove();
            const { showBar, moveBarH } = this.state;
            if (!showBar) {
                return;
            }
            const dom = e.currentTarget;
            const scrollMain = dom.firstElementChild;
            const startH = parseInt(scrollMain.style.top || "0", 0);
            let step = Math.sqrt(this.scrollMainH / this.scrollBoxH) * 14;
            step = e.deltaY < 0 ? step : -step;
            let h = -(startH + step);
            const maxH = this.scrollMainH - this.scrollBoxH;
            h = h < 0 ? 0 : h > maxH ? maxH : h;
            const factor = (this.scrollBoxH - moveBarH) / maxH;
            scrollMain.style.top = `${-h}px`;
            this.moveBar.current.style.top = `${h * factor}px`;
        };
        this.locateBar = (e) => {
            const { moveBarH } = this.state;
            let h = e.nativeEvent.offsetY;
            const dom = this.moveBar.current;
            const maxH = this.scrollBoxH - moveBarH;
            const factor = (this.scrollMainH - this.scrollBoxH) / maxH;
            const scrollMain = dom.parentElement
                .previousElementSibling;
            h = h < 0 ? 0 : h > maxH ? maxH : h;
            dom.style.top = `${h}px`;
            scrollMain.style.top = `${-h * factor}px`;
        };
        this.scrollMethods = {
            initScroll: this.upDateInit,
            scrollToTop: this.scrollToTop,
        };
        if (props.bindIntiScroll) {
            props.bindIntiScroll(this.scrollMethods);
        }
    }
    componentDidMount() {
        this.upDateInit(this.props.height);
        if (!this.props.noStopPageScroll) {
            this.moveBar.current.parentElement.parentElement.addEventListener("wheel", this.stopWhell, { passive: false });
        }
    }
    componentWillUnmount() {
        window.cancelAnimationFrame(this.timer);
        window.cancelAnimationFrame(this.upTimer);
        if (!this.props.noStopPageScroll) {
            this.moveBar.current.parentElement.parentElement.removeEventListener("wheel", this.stopWhell);
        }
    }
    animateScroll(start, end, callback) {
        if (this.timer) {
            window.cancelAnimationFrame(this.timer);
            this.timer = 0;
        }
        let time = 0;
        const timeEnd = Math.floor(Math.sqrt(Math.abs(end - start)));
        const fn = () => {
            this.timer = window.requestAnimationFrame(() => {
                const distance = String(tween.easeInCubic(time, start, end, timeEnd));
                this.moveBar.current.style.height = distance + "px";
                if (time < timeEnd) {
                    fn();
                }
                else {
                    this.timer = 0;
                    callback();
                }
                time++;
            });
        };
        fn();
    }
    componentDidUpdate(prevProps, prevState) {
        if (prevProps.height !== this.props.height) {
            this.upDateInit(this.props.height);
        }
        else if (this.state.showBar === prevState.showBar) {
            const { time } = this.props;
            const scrollMain = this.moveBar.current.parentElement
                .previousElementSibling;
            if (this.upTimer) {
                window.cancelAnimationFrame(this.upTimer);
                this.upTimer = 0;
            }
            this.upTimer = window.setTimeout(() => {
                const curScrollMainH = scrollMain.clientHeight;
                if (curScrollMainH !== this.scrollMainH) {
                    this.upDateInit(this.props.height);
                    this.upTimer = 0;
                }
            }, time);
        }
    }
    stopProp(e) {
        e.stopPropagation();
    }
    stopWhell(e) {
        e.preventDefault();
    }
    render() {
        const { className, height, children, keepBarShow } = this.props;
        const { showBar, moveBarH } = this.state;
        const style = height ? { height: `${height}px` } : undefined;
        return (React.createElement("div", { className: `g-scrollBox ${className}`, style: style, onMouseEnter: keepBarShow ? undefined : this.mouseOver, onMouseLeave: this.mouseLeave, onWheel: this.mainMousewheel, onMouseUp: this.cancelBarMove },
            React.createElement("div", { className: "m-scrollMain" }, children),
            React.createElement("div", { style: { display: showBar ? "block" : "none", }, className: "m-scrollBar", onClick: this.locateBar },
                React.createElement("div", { ref: this.moveBar, className: "m-moveBar", style: { height: `${moveBarH}px`, top: "0px", }, onClick: this.stopProp, onMouseDown: this.barClick }))));
    }
}
ScrollBox.defaultProps = {
    className: "",
};
export default ScrollBox;
