import * as React from "react";
import { Combobox } from "../combo/index";
import { SvgIcon } from "../my-icon/index";
const pageNumsArr = [
    { id: "1", text: "10" },
    { id: "2", text: "20" },
    { id: "3", text: "30" },
    { id: "4", text: "50" },
    { id: "5", text: "100" },
];
class PageSize extends React.PureComponent {
    constructor() {
        super(...arguments);
        this.pageCodeHandle = (e) => {
            const target = e.currentTarget;
            if (target.classList.contains("active")) {
                return;
            }
            const { changeHandle } = this.props;
            const num = ~~target.dataset.num;
            changeHandle("curPage", num);
        };
        this.curNumInpHandle = (e) => {
            const { totalPages, changeHandle } = this.props;
            let val = ~~e.currentTarget.value.trim();
            val = +val < 1 ? 1 : +val > totalPages ? totalPages : val;
            changeHandle("curPage", val);
        };
        this.controlBtnHandle = (e) => {
            const type = e.currentTarget.dataset.type;
            const { curPage, totalPages, changeHandle } = this.props;
            if (type === "pre") {
                changeHandle("curPage", Math.max(+curPage - 1, 1));
            }
            else {
                changeHandle("curPage", Math.min(+curPage + 1, totalPages));
            }
        };
        this.perNumComboClick = (selectedArr) => {
            this.props.changeHandle("perNums", +selectedArr[0].text);
            this.props.changeHandle("curPage", 1);
        };
    }
    firstPage() {
        const { totalPages, curPage } = this.props;
        const arr = Array.from({ length: 5 }, (...args) => args[1] + 1);
        const first = arr.map(val => (React.createElement("span", { className: "m-page-num " + (val === curPage ? "active" : ""), key: val, "data-num": val, onClick: this.pageCodeHandle }, val)));
        const className = "m-page-num " + (totalPages === curPage ? "active" : "");
        return (React.createElement(React.Fragment, null,
            first,
            React.createElement("span", null, "..."),
            React.createElement("span", { className: className, "data-num": totalPages, onClick: this.pageCodeHandle }, totalPages)));
    }
    lastPage() {
        const { totalPages, curPage } = this.props;
        const arr = Array.from({ length: 5 }, (...args) => args[1] + 1);
        const last = arr.map(val => {
            const className = "m-page-num " + (totalPages - 5 + val === curPage ? "active" : "");
            return (React.createElement("span", { className: className, "data-num": totalPages - 5 + val, onClick: this.pageCodeHandle, key: totalPages - 5 + val }, totalPages - 5 + val));
        });
        return (React.createElement(React.Fragment, null,
            React.createElement("span", { className: "m-page-num " + (1 === curPage ? "active" : ""), "data-num": 1, onClick: this.pageCodeHandle }, 1),
            React.createElement("span", null, "..."),
            last));
    }
    centerPage() {
        const { curPage, totalPages } = this.props;
        const arr = Array.from({ length: 5 }, (...args) => args[1] + 1);
        const center = arr.map(val => {
            const className = "m-page-num " + (curPage - 3 + val === curPage ? "active" : "");
            return (React.createElement("span", { "data-num": curPage - 3 + val, onClick: this.pageCodeHandle, className: className, key: curPage - 3 + val }, curPage - 3 + val));
        });
        const className = "m-page-num " + (totalPages === curPage ? "active" : "");
        return (React.createElement(React.Fragment, null,
            React.createElement("span", { className: "m-page-num " + (1 === curPage ? "active" : ""), "data-num": 1, onClick: this.pageCodeHandle }, 1),
            React.createElement("span", null, "..."),
            center,
            React.createElement("span", null, "..."),
            React.createElement("span", { className: className, "data-num": totalPages, onClick: this.pageCodeHandle }, totalPages)));
    }
    normalPage() {
        const { curPage, totalPages } = this.props;
        const arr = Array.from({ length: totalPages }, (...args) => args[1] + 1);
        return arr.map(val => {
            const className = "m-page-num " + (val === curPage ? "active" : "");
            return (React.createElement("span", { className: className, "data-num": val, onClick: this.pageCodeHandle, key: val }, val));
        });
    }
    render() {
        const { totalPages, curPage, totalNums, perNums } = this.props;
        let navigatepageCom;
        if (totalPages < 11) {
            navigatepageCom = this.normalPage();
        }
        else if (curPage - 1 < 4) {
            navigatepageCom = this.firstPage();
        }
        else if (totalPages - curPage < 4) {
            navigatepageCom = this.lastPage();
        }
        else {
            navigatepageCom = this.centerPage();
        }
        const perNumId = pageNumsArr.find(val => ~~val.text === perNums).id;
        return (React.createElement("div", { className: "g-pageCode" },
            React.createElement("div", { className: "g-pageLeft" },
                React.createElement("div", { className: "m-page-total" },
                    React.createElement("span", null,
                        "\u5171 ",
                        totalPages,
                        " \u9875"),
                    React.createElement("span", null,
                        totalNums,
                        "\u6761")),
                React.createElement("div", { style: { display: "flex", alignItems: "center", } },
                    React.createElement("span", null, "\u6BCF\u9875\u663E\u793A\uFF1A"),
                    React.createElement(Combobox, { field: "pageNums", data: pageNumsArr, width: 100, directionUp: true, clickCallback: this.perNumComboClick, defaultVal: perNumId }),
                    React.createElement("span", null, "\u00A0\u6761"))),
            React.createElement("div", { className: "g-pageRight" },
                React.createElement("div", { style: { marginRight: "20px", } },
                    React.createElement("span", null, "\u8DF3\u8F6C\u5230"),
                    React.createElement("input", { className: "j-jump-page s-inp normal", value: curPage, type: "number", min: 1, onChange: this.curNumInpHandle })),
                React.createElement("div", { className: "m-code-number" },
                    React.createElement("span", { className: "m-page-num", "data-type": "pre", onClick: this.controlBtnHandle },
                        React.createElement(SvgIcon, { className: "arrow-left" })),
                    React.createElement("span", null, navigatepageCom),
                    React.createElement("span", { className: "m-page-num", onClick: this.controlBtnHandle, "data-type": "next" },
                        React.createElement(SvgIcon, { className: "arrow-right" }))))));
    }
}
export default PageSize;
