import * as Immutable from "immutable";
var activeStatus;
(function (activeStatus) {
    activeStatus["select"] = "selected";
    activeStatus["hasSelect"] = "hasSelect";
    activeStatus["noSelect"] = "noSelect";
})(activeStatus || (activeStatus = {}));
const formatterTreeData = function (fixObj, defaultVal, data) {
    const { idField: id, childField, multiply } = fixObj;
    let defaultValArr = defaultVal.split(",");
    if (!multiply && defaultValArr.length > 1) {
        defaultValArr.length = 1;
    }
    let oldSelectedIndex = "";
    let listSelected = Immutable.List([]);
    const immutableData = Immutable.fromJS(data, function (_key, val, path) {
        if (Immutable.isKeyed(val)) {
            let node = val.toOrderedMap();
            let children = node.get(childField);
            if (!children) {
                children = Immutable.List([]);
                node = node.set(childField, children);
            }
            let active;
            if (children.size) {
                if (multiply) {
                    const hasSelected = children.some((val) => {
                        return (val.get("active") ===
                            activeStatus.hasSelect);
                    });
                    if (hasSelected) {
                        active = activeStatus.hasSelect;
                    }
                    else {
                        let selectCount = children.reduce((cur, val) => {
                            let total = cur;
                            if (val.get("active") ===
                                activeStatus.select) {
                                total++;
                            }
                            return total;
                        }, 0);
                        const childSize = children.size;
                        if (selectCount === childSize) {
                            active = activeStatus.select;
                        }
                        else if (selectCount === 0) {
                            active = activeStatus.noSelect;
                        }
                        else {
                            active = activeStatus.hasSelect;
                        }
                    }
                }
                else {
                    active = activeStatus.noSelect;
                }
            }
            else {
                const isDefault = defaultValArr.includes(`${node.get(id)}`);
                active = isDefault
                    ? activeStatus.select
                    : activeStatus.noSelect;
                if (isDefault) {
                    const idVal = node.get(id);
                    listSelected.push(idVal);
                    if (!multiply) {
                        oldSelectedIndex = path.join(",");
                    }
                }
            }
            node = node.withMutations(map => {
                return map.set("active", active).set("expand", true);
            });
            return node;
        }
        else {
            return val.toList();
        }
    });
    return {
        data: immutableData,
        oldSelectedIndex: oldSelectedIndex,
        selectArr: listSelected
    };
};
const cascade = function (path, data, childField) {
    let _data = data;
    const pathArr = path.split(',');
    pathArr.pop();
    _data = _data.withMutations(list => {
        let _list = list;
        while (pathArr.length) {
            const path = pathArr.join(`,${childField},`).split(",");
            _list = _list.updateIn(path, node => {
                let _node = node;
                const child = _node.get(childField);
                const hasSelect = child.some(val => val.get("active") === activeStatus.hasSelect);
                let active;
                if (hasSelect) {
                    active = activeStatus.hasSelect;
                }
                else {
                    const selectCount = child.reduce((cur, val) => {
                        let total = cur;
                        if (val.get("active") === activeStatus.select) {
                            total++;
                        }
                        return total;
                    }, 0);
                    const childSize = child.size;
                    if (selectCount === childSize) {
                        active = activeStatus.select;
                    }
                    else if (selectCount === 0) {
                        active = activeStatus.noSelect;
                    }
                    else {
                        active = activeStatus.hasSelect;
                    }
                }
                return _node.set("active", active);
            });
            pathArr.pop();
        }
        return _list;
    });
    return _data;
};
export { formatterTreeData, cascade, activeStatus };
